@@ -4,12 +4,10 @@
 #include <unordered_map>
 #include <unordered_set>
 
-#include <torch/csrc/WindowsTorchApiMacro.h>
+#include <torch/csrc/Export.h>
 #include <torch/csrc/jit/tensorexpr/fwd_decls.h>
 
-namespace torch {
-namespace jit {
-namespace tensorexpr {
+namespace torch::jit::tensorexpr {
 
 class VarHandle;
 class Var;
@@ -19,12 +17,11 @@ using VarNameMap = std::unordered_map<VarPtr, std::string>;
 // A manager to get unique names from vars.
 // It starts with the name hints of the var and append "_" + $counter until it
 // hits a unique name.
-// NOLINTNEXTLINE(cppcoreguidelines-pro-type-member-init)
 class TORCH_API UniqueNameManager {
  public:
   const std::string& get_unique_name(const VarHandle& v);
 
-  const std::string& get_unique_name(VarPtr v);
+  const std::string& get_unique_name(const VarPtr& v);
 
  private:
   friend class ScopedVarName;
@@ -33,6 +30,4 @@ class TORCH_API UniqueNameManager {
   std::unordered_set<std::string> all_unique_names_;
 };
 
-} // namespace tensorexpr
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit::tensorexpr
