@@ -79,7 +79,7 @@ def reset_counters() -> None:
     call_counter_debug_info.clear()
 
 
-@functools.lru_cache(None)
+@functools.cache
 def get_env_val(env_str: str) -> Optional[str]:
     return os.environ.get(env_str, None)
 
