@@ -301,7 +301,7 @@ class Transformer(Module):
         """
         return _generate_square_subsequent_mask(sz, dtype=dtype, device=device)
 
-    def _reset_parameters(self):
+    def _reset_parameters(self) -> None:
         r"""Initiate parameters in the transformer model."""
         for p in self.parameters():
             if p.dim() > 1:
